from django.apps import AppConfig


class SimpelSitesConfig(AppConfig):
    icon = "web"
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_routes"
    label = "django_routes"
    verbose_name = "Django Routes"
